import logging
from abc import abstractmethod, ABCMeta

import git
import validators
from git import Repo

from falconcv.models.clone_progress import CloneProgress
from falconcv.util import LibUtils

logger = logging.getLogger(__name__)


class ApiInstaller(metaclass=ABCMeta):
    @abstractmethod
    def __init__(self, repo_uri):
        self.repo_uri = repo_uri
        self.package_name = None
        self.repo_folder = None

    @staticmethod
    def get_repo_name_from_url(url: str) -> str:
        last_slash_index = url.rfind("/")
        last_suffix_index = url.rfind(".git")
        if last_suffix_index < 0:
            last_suffix_index = len(url)
        if last_slash_index < 0 or last_suffix_index <= last_slash_index:
            raise Exception("invalid url format {}".format(url))
        return url[last_slash_index + 1 : last_suffix_index]

    def __clone_repo(self):
        try:
            assert self.repo_uri and validators.url(self.repo_uri), "Invalid repo uri"
            repo_name = self.get_repo_name_from_url(self.repo_uri)
            self.repo_folder = LibUtils.repos_home().joinpath(repo_name)
            self.repo_folder.mkdir(exist_ok=True)
            if not self.repo_folder.joinpath(".git").is_dir():
                Repo.clone_from(
                    self.repo_uri,
                    self.repo_folder,
                    depth=1,
                    branch="master",
                    progress=CloneProgress(repo_name),
                )
        except git.exc.InvalidGitRepositoryError as ex:
            logger.error(
                "Error cloning the repository from {} : {}".format(self.repo_uri, ex)
            )
        except git.exc.GitError as ex:
            logger.error(
                "Error cloning the repository from {} : {}".format(self.repo_uri, ex)
            )

    @abstractmethod
    def install(self):
        self.__clone_repo()
