from falconcv.decor import exception
from enum import Enum, auto


class ModelBackend(Enum):
    TENSORFLOW = auto()
    PYTORCH = auto()


class ModelBuilder:
    @classmethod
    @exception
    def build(
        cls, model=None, labels_map=None, config=None, backend=ModelBackend.TENSORFLOW
    ):
        # create model base on the parameters
        if not config and not model:
            raise Exception("Invalid model argument")
        if backend == ModelBackend.TENSORFLOW:
            from falconcv.models.tf import APIModelFactory

            return APIModelFactory.create(model, labels_map, config)
        elif backend == ModelBackend.PYTORCH:
            raise NotImplementedError("Not implemented yet")
        else:
            raise NotImplementedError("Invalid backend parameter")
