import importlib
import logging
import os
import subprocess
from pathlib import Path

from falconcv.models import ApiInstaller
from falconcv.util import FileUtils

logger = logging.getLogger(__name__)


def _copy(self, target):
    import shutil

    assert self.is_file()
    shutil.copy(str(self), str(target))  # str() only there for Python < (3, 6)


Path.copy = _copy

TFODAPI_URI = "https://github.com/tensorflow/models.git"


class TFObjectDetectionAPI(ApiInstaller):
    def __init__(self):
        super(TFObjectDetectionAPI, self).__init__(TFODAPI_URI)
        self._package_name = "object_detection"

    def install(self):
        try:
            super(TFObjectDetectionAPI, self).install()
            self._protobuf_comp()
            research_folder = self.repo_folder.joinpath("research")
            od_folder = research_folder.joinpath("object_detection")
            setup_package_file: Path = od_folder.joinpath("packages/tf2/setup.py")
            test_file = od_folder.joinpath("builders/model_builder_tf2_test.py")
            # slim_folder = research_folder.joinpath("slim")
            if importlib.util.find_spec(self._package_name) is None:
                logger.debug("Installing Api")
                with FileUtils.workon(str(research_folder)):
                    if setup_package_file.exists():
                        print(os.getcwd())
                        setup_package_file.copy(".")
                        os.system("python -m pip install .")
                        # os.system("python -m pip install --use-feature=2020-resolver .")
                logger.debug("Api installation done")
            # os.system(f"python {test_file}")
            # sys.path.append(str(research_folder))
            # sys.path.append(str(slim_folder))
            # os.environ['PATH'] += "{}{}{}".format(str(research_folder), os.pathsep, str(slim_folder))
        except Exception as ex:
            print(ex)
            logger.error("Error installing the package : {}".format(ex))

    def _protobuf_comp(self):
        research_folder = self.repo_folder.joinpath("research")
        protos_folder = research_folder.joinpath("object_detection", "protos")
        protos_files = Path(protos_folder).rglob("*.proto")
        for proto_file in protos_files:
            file_name = proto_file.name
            rel_file_path = "object_detection/protos/{}".format(file_name)
            p = subprocess.Popen(
                ["protoc", rel_file_path, "--python_out=."],
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                universal_newlines=True,
                cwd=str(research_folder),
            )
            output = p.stdout.readlines()
            error = p.stderr.readlines()
            if error:
                print(error)
                raise IOError(error)
            if output:
                print(output)
                logger.debug(output)
            p.wait()


if __name__ == "__main__":
    api = TFObjectDetectionAPI()
    api.install()
