from falconcv.util import ColorUtils
import numpy as np
import matplotlib.pylab as plt
import math
from matplotlib import patches
from PIL import Image


class VisUtils:
    @staticmethod
    def draw_mask(image, mask, color=(255, 0, 0), alpha=0.4):
        assert image.dtype == np.uint8, "`image` not of type np.uint8"
        assert mask.dtype == np.uint8, "`mask` not of type np.uint8"
        if np.any(np.logical_and(mask != 1, mask != 0)):
            raise ValueError("`mask` elements should be in [0, 1]")
        if image.shape[:2] != mask.shape:
            raise ValueError(
                "The image has spatial dimensions %s but the mask has "
                "dimensions %s" % (image.shape[:2], mask.shape)
            )
        pil_image = Image.fromarray(image)
        solid_color = np.expand_dims(np.ones_like(mask), axis=2) * np.reshape(
            list(color), [1, 1, 3]
        )
        pil_solid_color = Image.fromarray(np.uint8(solid_color)).convert("RGBA")
        pil_mask = Image.fromarray(np.uint8(255.0 * alpha * mask)).convert("L")
        pil_image = Image.composite(pil_solid_color, pil_image, pil_mask)
        np.copyto(image, np.array(pil_image.convert("RGB")))

    @classmethod
    def imshow(cls, img, boxes, alpha=0.3):
        labels = np.unique([box.label for box in boxes])
        colors_dict = ColorUtils.rainbow(len(labels))
        R = np.asarray(colors_dict["r"], dtype=np.float) / 255.0
        G = np.asarray(colors_dict["g"], dtype=np.float) / 255.0
        B = np.asarray(colors_dict["b"], dtype=np.float) / 255.0
        A = [1.0] * len(labels)
        palette = list(zip(B.tolist(), G.tolist(), R.tolist(), A))
        colors = {label: palette[i] for i, label in enumerate(labels)}
        fig = plt.figure(figsize=(20, 10))
        ax = fig.add_subplot(111, aspect="equal")
        for i, box in enumerate(boxes):
            c = colors[box.label]
            if isinstance(box.mask, np.ndarray):
                mask = box.mask
                rgb_color = tuple(map(lambda x: math.floor(x * 255), c[:-1]))
                cls.draw_mask(img, mask, color=rgb_color, alpha=alpha)
            label = "{} :{:.2f}".format(box.label, box.score)
            ax.add_patch(
                patches.Rectangle(
                    (box.x1, box.y1),
                    box.x2 - box.x1,
                    box.y2 - box.y1,
                    linewidth=3,
                    edgecolor=c,
                    facecolor="none",
                    fill=False,
                )
            )
            ax.text(
                x=box.x1,
                y=box.y1,
                s=label,
                color="white",
                fontsize=12,
                bbox=dict(boxstyle="round", facecolor=colors[box.label], alpha=0.9),
            )
            ax.set_axis_off()
        ax.imshow(img)
        plt.show()
        return fig
